#
# Copyright © 2024 Charles E. Vejnar
#
# This is free software, licensed under the GNU General Public License v3.
# See /LICENSE for more information.
#

"""TargetScan features."""

from dataclasses import dataclass


@dataclass
class TargetscanTSType:
    """TargetScan parameters for a target site type
    # tgs_au_up_shift and tgs_au_down_shift are set in reference to end_site
    # ca_ are parameters for tgs_au
    # po_ ------------------ tgs_position
    # pa_ ------------------ tgs_pairing3p
    """

    name: str
    up_shift: int
    down_shift: int
    fc_mean: float
    ca_fc_slope: float
    ca_fc_intercept: float
    ca_weights_up: list[float]
    ca_weights_down: list[float]
    po_fc_slope: float
    po_fc_intercept: float
    pa_fc_slope: float
    pa_fc_intercept: float
    pa_mirna_seed_start: int
    pa_mirna_seed_overhang: int


ts_types = {}
ts_types["6mer"] = TargetscanTSType(
    "6mer",
    -8,
    0,
    -0.015,
    -0.241,
    0.115,
    [
        31.0,
        30.0,
        29.0,
        28.0,
        27.0,
        26.0,
        25.0,
        24.0,
        23.0,
        22.0,
        21.0,
        20.0,
        19.0,
        18.0,
        17.0,
        16.0,
        15.0,
        14.0,
        13.0,
        12.0,
        11.0,
        10.0,
        9.0,
        8.0,
        7.0,
        6.0,
        5.0,
        4.0,
        3.0,
        2.0,
    ],
    [
        2.0,
        2.0,
        3.0,
        4.0,
        5.0,
        6.0,
        7.0,
        8.0,
        9.0,
        10.0,
        11.0,
        12.0,
        13.0,
        14.0,
        15.0,
        16.0,
        17.0,
        18.0,
        19.0,
        20.0,
        21.0,
        22.0,
        23.0,
        24.0,
        25.0,
        26.0,
        27.0,
        28.0,
        29.0,
        30.0,
    ],
    0.000049,
    -0.033,
    -0.00278,
    -0.0091,
    7,
    1,
)
ts_types["7mer-A1"] = TargetscanTSType(
    "7mer-A1",
    -8,
    1,
    -0.099,
    -0.42,
    0.137,
    [
        31.0,
        30.0,
        29.0,
        28.0,
        27.0,
        26.0,
        25.0,
        24.0,
        23.0,
        22.0,
        21.0,
        20.0,
        19.0,
        18.0,
        17.0,
        16.0,
        15.0,
        14.0,
        13.0,
        12.0,
        11.0,
        10.0,
        9.0,
        8.0,
        7.0,
        6.0,
        5.0,
        4.0,
        3.0,
        2.0,
    ],
    [
        2.0,
        3.0,
        4.0,
        5.0,
        6.0,
        7.0,
        8.0,
        9.0,
        10.0,
        11.0,
        12.0,
        13.0,
        14.0,
        15.0,
        16.0,
        17.0,
        18.0,
        19.0,
        20.0,
        21.0,
        22.0,
        23.0,
        24.0,
        25.0,
        26.0,
        27.0,
        28.0,
        29.0,
        30.0,
        31.0,
    ],
    0.000072,
    -0.131,
    -0.0211,
    -0.053,
    7,
    1,
)
ts_types["7mer-m8"] = TargetscanTSType(
    "7mer-m8",
    -8,
    0,
    -0.161,
    -0.5,
    0.108,
    [
        30.0,
        29.0,
        28.0,
        27.0,
        26.0,
        25.0,
        24.0,
        23.0,
        22.0,
        21.0,
        20.0,
        19.0,
        18.0,
        17.0,
        16.0,
        15.0,
        14.0,
        13.0,
        12.0,
        11.0,
        10.0,
        9.0,
        8.0,
        7.0,
        6.0,
        5.0,
        4.0,
        3.0,
        2.0,
        1.0,
    ],
    [
        2.0,
        2.0,
        3.0,
        4.0,
        5.0,
        6.0,
        7.0,
        8.0,
        9.0,
        10.0,
        11.0,
        12.0,
        13.0,
        14.0,
        15.0,
        16.0,
        17.0,
        18.0,
        19.0,
        20.0,
        21.0,
        22.0,
        23.0,
        24.0,
        25.0,
        26.0,
        27.0,
        28.0,
        29.0,
        30.0,
    ],
    0.000091,
    -0.198,
    -0.031,
    -0.094,
    8,
    0,
)
ts_types["8mer"] = TargetscanTSType(
    "8mer",
    -8,
    1,
    -0.31,
    -0.64,
    0.055,
    [
        30.0,
        29.0,
        28.0,
        27.0,
        26.0,
        25.0,
        24.0,
        23.0,
        22.0,
        21.0,
        20.0,
        19.0,
        18.0,
        17.0,
        16.0,
        15.0,
        14.0,
        13.0,
        12.0,
        11.0,
        10.0,
        9.0,
        8.0,
        7.0,
        6.0,
        5.0,
        4.0,
        3.0,
        2.0,
        1.0,
    ],
    [
        2.0,
        3.0,
        4.0,
        5.0,
        6.0,
        7.0,
        8.0,
        9.0,
        10.0,
        11.0,
        12.0,
        13.0,
        14.0,
        15.0,
        16.0,
        17.0,
        18.0,
        19.0,
        20.0,
        21.0,
        22.0,
        23.0,
        24.0,
        25.0,
        26.0,
        27.0,
        28.0,
        29.0,
        30.0,
        31.0,
    ],
    0.000172,
    -0.38,
    -0.0041,
    -0.299,
    8,
    0,
)
